@@ -15,12 +15,14 @@
 #include <cairo-pdf.h>
 #endif
 #include <goocanvas.h>
+#include "demo-item.h"
 
 GooCanvasItem *ellipse2, *textitem;
 
 gboolean dragging = FALSE;
 double drag_x, drag_y;
 
+static void setup_canvas (GooCanvas *canvas);
 GtkWidget *create_canvas_fifteen (void);
 GtkWidget *create_canvas_features (void);
 GtkWidget *create_canvas_arrowhead (void);
@@ -29,10 +31,12 @@ GtkWidget *create_grabs_page (void);
 GtkWidget *create_events_page (void);
 GtkWidget *create_paths_page (void);
 GtkWidget *create_focus_page (void);
+GtkWidget *create_animation_page (void);
+GtkWidget *create_clipping_page (void);
 
 #if CAIRO_HAS_PDF_SURFACE
 static void
-write_pdf_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+write_pdf_clicked (GtkWidget *button, GooCanvas *canvas)
 {
   cairo_surface_t *surface;
   cairo_t *cr;
@@ -45,7 +49,7 @@ write_pdf_clicked (GtkWidget *button, GooCanvasView *canvas_view)
   /* Place it in the middle of our 9x10 page. */
   cairo_translate (cr, 20, 130);
 
-  goo_canvas_view_render (canvas_view, cr, NULL, 1.0);
+  goo_canvas_render (canvas, cr, NULL, 1.0);
 
   cairo_show_page (cr);
 
@@ -55,9 +59,9 @@ write_pdf_clicked (GtkWidget *button, GooCanvasView *canvas_view)
 #endif
 
 static void
-zoom_changed (GtkAdjustment *adj, GooCanvasView *canvas_view)
+zoom_changed (GtkAdjustment *adj, GooCanvas *canvas)
 {
-  goo_canvas_view_set_scale (canvas_view, adj->value);
+  goo_canvas_set_scale (canvas, adj->value);
 }
 
 
@@ -70,61 +74,61 @@ center_toggled (GtkToggleButton *button, gpointer data)
 
 
 static void
-anchor_toggled (GtkWidget *button, GooCanvasView *canvas_view)
+anchor_toggled (GtkWidget *button, GooCanvas *canvas)
 {
   GtkAnchorType anchor;
 
   anchor = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "anchor"));
 
   if (GTK_TOGGLE_BUTTON (button)->active)
-    g_object_set (canvas_view, "anchor", anchor, NULL);
+    g_object_set (canvas, "anchor", anchor, NULL);
 }
 
 
 static void
-scroll_to_50_50_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+scroll_to_50_50_clicked (GtkWidget *button, GooCanvas *canvas)
 {
-  goo_canvas_view_scroll_to (canvas_view, 50, 50);
+  goo_canvas_scroll_to (canvas, 50, 50);
 }
 
 
 static void
-scroll_to_500_500_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+scroll_to_500_500_clicked (GtkWidget *button, GooCanvas *canvas)
 {
-  goo_canvas_view_scroll_to (canvas_view, 500, 500);
+  goo_canvas_scroll_to (canvas, 500, 500);
 }
 
 
 static void
-scroll_to_250_250_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+scroll_to_250_250_clicked (GtkWidget *button, GooCanvas *canvas)
 {
-  goo_canvas_view_scroll_to (canvas_view, 250, 250);
+  goo_canvas_scroll_to (canvas, 250, 250);
 }
 
 
 static void
-animate_ellipse_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+animate_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 {
 #if 1
-  goo_canvas_item_animate (ellipse2, 100, 100, 1, 90, 1000, 40,
+  goo_canvas_item_animate (ellipse2, 100, 100, 1, 90, TRUE, 1000, 40,
 			   GOO_CANVAS_ANIMATE_BOUNCE);
 #endif
 #if 0
-  goo_canvas_item_animate (textitem, -300, -200, 1, 90, 1000, 40,
+  goo_canvas_item_animate (textitem, -300, -200, 1, 90, TRUE, 1000, 40,
 			   GOO_CANVAS_ANIMATE_BOUNCE);
 #endif
 }
 
 
 static void
-stop_animation_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+stop_animation_clicked (GtkWidget *button, GooCanvas *canvas)
 {
   goo_canvas_item_stop_animation (ellipse2);
 }
 
 
 static void
-move_ellipse_clicked (GtkWidget *button, GooCanvasView *canvas_view)
+move_ellipse_clicked (GtkWidget *button, GooCanvas *canvas)
 {
   static int last_state = 0;
 
@@ -162,6 +166,31 @@ move_ellipse_clicked (GtkWidget *button, GooCanvasView *canvas_view)
 		    NULL);
       last_state = 2;
     }
+  else if (last_state == 2)
+    {
+      g_object_set (ellipse2,
+		    "center-x", 0.0,
+		    "center-y", 0.0,
+		    "radius-y", 30.0,
+		    NULL);
+      goo_canvas_item_set_simple_transform (ellipse2, 100, 100, 1, 0);
+      last_state = 3;
+    }
+  else if (last_state == 3)
+    {
+      goo_canvas_item_set_simple_transform (ellipse2, 200, 100, 2, 0);
+      last_state = 4;
+    }
+  else if (last_state == 4)
+    {
+      goo_canvas_item_set_simple_transform (ellipse2, 200, 200, 1, 45);
+      last_state = 5;
+    }
+  else if (last_state == 5)
+    {
+      goo_canvas_item_set_simple_transform (ellipse2, -50, -50, 0.2, 225);
+      last_state = 6;
+    }
   else
     {
       g_object_set (ellipse2,
@@ -180,13 +209,11 @@ move_ellipse_clicked (GtkWidget *button, GooCanvasView *canvas_view)
 
 
 static gboolean
-on_motion_notify (GooCanvasItemView *view,
-		  GooCanvasItemView *target,
+on_motion_notify (GooCanvasItem *item,
+		  GooCanvasItem *target,
 		  GdkEventMotion *event,
 		  gpointer data)
 {
-  GooCanvasItem *item = goo_canvas_item_view_get_item (view);
-
 #if 0
   g_print ("received 'motion-notify' signal at %g, %g\n",
 	   event->x, event->y);
@@ -205,18 +232,17 @@ on_motion_notify (GooCanvasItemView *view,
 
 
 static gboolean
-on_button_press (GooCanvasItemView *view,
-		 GooCanvasItemView *target,
+on_button_press (GooCanvasItem *item,
+		 GooCanvasItem *target,
 		 GdkEventButton *event,
 		 gpointer data)
 {
-  GooCanvasItem *item = goo_canvas_item_view_get_item (view);
-  GooCanvasView *canvas_view;
+  GooCanvas *canvas;
   GdkCursor *fleur;
 
 #if 0
   g_print ("%p received 'button-press' signal at %g, %g (root: %g, %g)\n",
-	   view, event->x, event->y, event->x_root, event->y_root);
+	   item, event->x, event->y, event->x_root, event->y_root);
 #endif
 
   switch (event->button)
@@ -237,11 +263,11 @@ on_button_press (GooCanvasItemView *view,
 	  drag_y = event->y;
 
 	  fleur = gdk_cursor_new (GDK_FLEUR);
-	  canvas_view = goo_canvas_item_view_get_canvas_view (view);
-	  goo_canvas_view_pointer_grab (canvas_view, view,
-					GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-					fleur,
-					event->time);
+	  canvas = goo_canvas_item_get_canvas (item);
+	  goo_canvas_pointer_grab (canvas, item,
+				   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+				   fleur,
+				   event->time);
 	  gdk_cursor_unref (fleur);
 	  dragging = TRUE;
 	}
@@ -264,19 +290,19 @@ on_button_press (GooCanvasItemView *view,
 
 
 static gboolean
-on_button_release (GooCanvasItemView *view,
-		   GooCanvasItemView *target,
+on_button_release (GooCanvasItem *item,
+		   GooCanvasItem *target,
 		   GdkEventButton *event,
 		   gpointer data)
 {
-  GooCanvasView *canvas_view;
+  GooCanvas *canvas;
 
 #if 0
   g_print ("received 'button-release' signal\n");
 #endif
 
-  canvas_view = goo_canvas_item_view_get_canvas_view (view);
-  goo_canvas_view_pointer_ungrab (canvas_view, view, event->time);
+  canvas = goo_canvas_item_get_canvas (item);
+  goo_canvas_pointer_ungrab (canvas, item, event->time);
   dragging = FALSE;
 
   return TRUE;
@@ -284,13 +310,13 @@ on_button_release (GooCanvasItemView *view,
 
 
 static gboolean
-on_background_button_press (GooCanvasItemView *view,
-			    GooCanvasItemView *target,
+on_background_button_press (GooCanvasItem *item,
+			    GooCanvasItem *target,
 			    GdkEventButton *event,
 			    gpointer data)
 {
 #if 1
-  g_print ("background item view received 'button-press' signal\n");
+  g_print ("background received 'button-press' signal\n");
 #endif
 
   return TRUE;
@@ -298,35 +324,24 @@ on_background_button_press (GooCanvasItemView *view,
 
 
 static void
-on_item_view_created (GooCanvasView     *view,
-		      GooCanvasItemView *item_view,
-		      GooCanvasItem     *item,
-		      gpointer           data)
+setup_item_signals (GooCanvasItem *item)
 {
-  if (!goo_canvas_item_get_parent (item))
-    {
-      g_signal_connect (item_view, "button_press_event",
-			(GtkSignalFunc) on_background_button_press, NULL);
-    }
-  else if (!g_object_get_data (G_OBJECT (item), "skip-signal-connection"))
-    {
-      g_signal_connect (item_view, "motion_notify_event",
-			(GtkSignalFunc) on_motion_notify, NULL);
-      g_signal_connect (item_view, "button_press_event",
-			(GtkSignalFunc) on_button_press, NULL);
-      g_signal_connect (item_view, "button_release_event",
-			(GtkSignalFunc) on_button_release, NULL);
-    }
+  g_signal_connect (item, "motion_notify_event",
+		    (GtkSignalFunc) on_motion_notify, NULL);
+  g_signal_connect (item, "button_press_event",
+		    (GtkSignalFunc) on_button_press, NULL);
+  g_signal_connect (item, "button_release_event",
+		    (GtkSignalFunc) on_button_release, NULL);
 }
 
 
 GtkWidget *
-create_canvas_primitives (GooCanvasModelSimple *canvas_model)
+create_canvas_primitives ()
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *w;
-	GtkWidget *scrolled_win, *canvas_view;
+	GtkWidget *scrolled_win, *canvas;
 	GtkAdjustment *adj;
 	GSList *group = NULL;
 	GdkColor bg_color = { 0, 50000, 50000, 65535 };
@@ -345,16 +360,9 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 
 	/* Create the canvas */
 
-	canvas_view = goo_canvas_view_new ();
-
-	gtk_widget_modify_base (canvas_view, GTK_STATE_NORMAL, &bg_color);
-
-	g_signal_connect (canvas_view, "item_view_created",
-			  (GtkSignalFunc) on_item_view_created,
-			  NULL);
-
-	goo_canvas_view_set_bounds (GOO_CANVAS_VIEW (canvas_view),
-				    0, 0, 604, 454);
+	canvas = goo_canvas_new ();
+	gtk_widget_modify_base (canvas, GTK_STATE_NORMAL, &bg_color);
+	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 604, 454);
 
 	/* Zoom */
 
@@ -366,7 +374,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
 			  (GtkSignalFunc) zoom_changed,
-			  canvas_view);
+			  canvas);
 	gtk_widget_set_size_request (w, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
@@ -378,7 +386,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	/*gtk_widget_show (w);*/
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) center_toggled,
-			  canvas_view);
+			  canvas);
 
 	/* Move Ellipse */
 
@@ -387,7 +395,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) move_ellipse_clicked,
-			  canvas_view);
+			  canvas);
 
 	/* Animate Ellipse */
 
@@ -396,7 +404,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) animate_ellipse_clicked,
-			  canvas_view);
+			  canvas);
 	
 	/* Stop Animation */
 
@@ -405,7 +413,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) stop_animation_clicked,
-			  canvas_view);
+			  canvas);
 	
 
 #if CAIRO_HAS_PDF_SURFACE
@@ -416,7 +424,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) write_pdf_clicked,
-			  canvas_view);
+			  canvas);
 #endif
 
 	hbox = gtk_hbox_new (FALSE, 4);
@@ -434,21 +442,21 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) scroll_to_50_50_clicked,
-			  canvas_view);
+			  canvas);
 
 	w = gtk_button_new_with_label("250,250");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) scroll_to_250_250_clicked,
-			  canvas_view);
+			  canvas);
 
 	w = gtk_button_new_with_label("500,500");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
 			  (GtkSignalFunc) scroll_to_500_500_clicked,
-			  canvas_view);
+			  canvas);
 
 	/* Scroll anchor */
 
@@ -462,7 +470,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NW));
 
@@ -472,7 +480,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_N));
 
@@ -482,7 +490,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NE));
 
@@ -492,7 +500,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_W));
 
@@ -502,7 +510,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_CENTER));
 
@@ -512,7 +520,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_E));
 
@@ -522,7 +530,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SW));
 
@@ -532,7 +540,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_S));
 
@@ -542,7 +550,7 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
 			  (GtkSignalFunc) anchor_toggled,
-			  canvas_view);
+			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SE));
 
@@ -553,14 +561,11 @@ create_canvas_primitives (GooCanvasModelSimple *canvas_model)
 	gtk_widget_show (scrolled_win);
 	gtk_box_pack_start (GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 0);
 
-	gtk_widget_show (canvas_view);
-	gtk_container_add (GTK_CONTAINER (scrolled_win), canvas_view);
+	gtk_widget_show (canvas);
+	gtk_container_add (GTK_CONTAINER (scrolled_win), canvas);
 
-	/* Set the model on the canvas view. The canvas view will then create
-	   item views to render each of the items. */
-	goo_canvas_view_set_model (GOO_CANVAS_VIEW (canvas_view),
-				   GOO_CANVAS_MODEL (canvas_model));
-	g_object_unref (canvas_model);
+	/* Add all the canvas items. */
+	setup_canvas (GOO_CANVAS (canvas));
 
 #if 0
 	g_signal_connect_after (canvas, "key_press_event",
@@ -594,36 +599,29 @@ static void
 setup_divisions (GooCanvasItem *root)
 {
   GooCanvasItem *group, *item;
-  GooCanvasStyle *style;
 
-  group = goo_canvas_group_new (root);
+  group = goo_canvas_group_new (root, NULL);
   goo_canvas_item_translate (group, 2, 2);
 
-  style = goo_canvas_style_new ();
-  style->mask = GOO_CANVAS_STYLE_LINE_WIDTH;
-  style->line_width = 4;
+  item = goo_canvas_rect_new (group, 0, 0, 600, 450,
+			      "line-width", 4.0,
+			      NULL);
 
-  item = goo_canvas_rect_new (group, 0, 0, 600, 450, NULL);
-  g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-  goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
+  item = goo_canvas_polyline_new_line (group, 0, 150, 600, 150,
+			      "line-width", 4.0,
+			      NULL);
 
-  item = goo_canvas_polyline_new_line (group, 0, 150, 600, 150, NULL);
-  g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-  goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
+  item = goo_canvas_polyline_new_line (group, 0, 300, 600, 300,
+			      "line-width", 4.0,
+			      NULL);
 
-  item = goo_canvas_polyline_new_line (group, 0, 300, 600, 300, NULL);
-  g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-  goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
+  item = goo_canvas_polyline_new_line (group, 200, 0, 200, 450,
+			      "line-width", 4.0,
+			      NULL);
 
-  item = goo_canvas_polyline_new_line (group, 200, 0, 200, 450, NULL);
-  g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-  goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
-
-  item = goo_canvas_polyline_new_line (group, 400, 0, 400, 450, NULL);
-  g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-  goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
-
-  goo_canvas_style_unref (style);
+  item = goo_canvas_polyline_new_line (group, 400, 0, 400, 450,
+			      "line-width", 4.0,
+			      NULL);
 
 #if 1
   setup_heading (group, "Rectangles", 0);
@@ -674,6 +672,7 @@ setup_rectangles (GooCanvasItem *root)
 			      "stroke-color", "red",
 			      "line-width", 8.0,
 			      NULL);
+  setup_item_signals (item);
 
   pattern = create_stipple ("mediumseagreen", stipple_data);
   item = goo_canvas_rect_new (root, 90, 40, 90, 60,
@@ -682,16 +681,19 @@ setup_rectangles (GooCanvasItem *root)
 			      "line-width", 4.0,
 			      NULL);
   cairo_pattern_destroy (pattern);
+  setup_item_signals (item);
 
   item = goo_canvas_rect_new (root, 10, 80, 70, 60,
 			      "fill-color", "steelblue",
 			      NULL);
+  setup_item_signals (item);
 
   item = goo_canvas_rect_new (root, 20, 90, 70, 60,
 			      "fill-color-rgba", 0x3cb37180,
 			      "stroke-color", "blue",
 			      "line-width", 2.0,
 			      NULL);
+  setup_item_signals (item);
 
   item = goo_canvas_rect_new (root, 110, 80, 50, 30,
 			      "radius-x", 20.0,
@@ -699,6 +701,12 @@ setup_rectangles (GooCanvasItem *root)
 			      "stroke-color", "yellow",
 			      "fill-color-rgba", 0x3cb3f180,
 			      NULL);
+  setup_item_signals (item);
+
+  item = goo_demo_item_new (root, 30, 20, 50, 30,
+			    "fill-color", "yellow",
+			    NULL);
+  setup_item_signals (item);
 }
 
 
@@ -712,10 +720,13 @@ setup_ellipses (GooCanvasItem *root)
     0, 0, 0, 0,     0, 0, 0, 255
   };
 
+#if 1
   ellipse1 = goo_canvas_ellipse_new (root, 245, 45, 25, 15,
 				     "stroke-color", "goldenrod",
 				     "line-width", 8.0,
 				     NULL);
+  setup_item_signals (ellipse1);
+#endif
 
   ellipse2 = goo_canvas_ellipse_new (root, 335, 70, 45, 30,
 				     "fill-color", "wheat",
@@ -724,9 +735,11 @@ setup_ellipses (GooCanvasItem *root)
 				     "title", "An ellipse",
 				     NULL);
 #if 0
-  goo_canvas_item_rotate (ellipse2, 45, 335, 70);
+  goo_canvas_item_rotate (ellipse2, 30, 0, 0);
 #endif
+  setup_item_signals (ellipse2);
 
+#if 1
   pattern = create_stipple ("cadetblue", stipple_data);
   ellipse3 = goo_canvas_ellipse_new (root, 245, 110, 35, 30,
 				     "fill-pattern", pattern,
@@ -734,13 +747,7 @@ setup_ellipses (GooCanvasItem *root)
 				     "line-width", 1.0,
 				     NULL);
   cairo_pattern_destroy (pattern);
-
-
-#if 0
- {
-   GooCanvasItem *clone = goo_canvas_clone_new (root, ellipse1);
-   goo_canvas_item_translate (clone, 100, 100);
- }
+  setup_item_signals (ellipse3);
 #endif
 }
 
@@ -752,19 +759,15 @@ static void
 polish_diamond (GooCanvasItem *root)
 {
   GooCanvasItem *group, *item;
-  GooCanvasStyle *style;
   int i, j;
   double a, x1, y1, x2, y2;
 
-  style = goo_canvas_style_new ();
-  style->mask = GOO_CANVAS_STYLE_STROKE_PATTERN | GOO_CANVAS_STYLE_LINE_WIDTH
-    | GOO_CANVAS_STYLE_LINE_CAP;
-  style->stroke_pattern = cairo_pattern_create_rgb (0, 0, 0);
-  style->line_width = 1;
-  style->line_cap = CAIRO_LINE_CAP_ROUND;
-
-  group = goo_canvas_group_new (root);
+  group = goo_canvas_group_new (root,
+				"line-width", 1.0,
+				"line-cap", CAIRO_LINE_CAP_ROUND,
+				NULL);
   goo_canvas_item_translate (group, 270, 230);
+  setup_item_signals (group);
 
   for (i = 0; i < VERTICES; i++) {
     a = 2.0 * M_PI * i / VERTICES;
@@ -776,12 +779,8 @@ polish_diamond (GooCanvasItem *root)
       x2 = RADIUS * cos (a);
       y2 = RADIUS * sin (a);
       item = goo_canvas_polyline_new_line (group, x1, y1, x2, y2, NULL);
-      g_object_set_data (G_OBJECT (item), "skip-signal-connection", "TRUE");
-      goo_canvas_item_simple_set_style (GOO_CANVAS_ITEM_SIMPLE (item), style);
     }
   }
-
-  goo_canvas_style_unref (style);
 }
 
 
@@ -829,14 +828,15 @@ make_hilbert (GooCanvasItem *root)
     }
 
   pattern = create_stipple ("red", stipple_data);
-  item = goo_canvas_item_new (root, GOO_TYPE_CANVAS_POLYLINE,
-			      "points", points,
-			      "line-width", 4.0,
-			      "stroke-pattern", pattern,
-			      "line-cap", CAIRO_LINE_CAP_SQUARE,
-			      "line-join", CAIRO_LINE_JOIN_MITER,
-			      NULL);
+  item = goo_canvas_polyline_new (root, FALSE, 0,
+				  "points", points,
+				  "line-width", 4.0,
+				  "stroke-pattern", pattern,
+				  "line-cap", CAIRO_LINE_CAP_SQUARE,
+				  "line-join", CAIRO_LINE_JOIN_MITER,
+				  NULL);
   cairo_pattern_destroy (pattern);
+  setup_item_signals (item);
 
   goo_canvas_points_unref (points);
 }
@@ -844,7 +844,7 @@ make_hilbert (GooCanvasItem *root)
 static void
 setup_lines (GooCanvasItem *root)
 {
-  GooCanvasItem *polyline1, *polyline2, *polyline3;
+  GooCanvasItem *polyline1, *polyline2, *polyline3, *polyline4, *polyline5;
 
   polish_diamond (root);
   make_hilbert (root);
@@ -864,6 +864,7 @@ setup_lines (GooCanvasItem *root)
 				       "arrow-length", 4.0,
 				       "arrow-width", 3.5,
 				       NULL);
+  setup_item_signals (polyline1);
 
   polyline2 = goo_canvas_polyline_new (root, FALSE, 2,
 				       356.0, 180.0,
@@ -876,6 +877,7 @@ setup_lines (GooCanvasItem *root)
 				       "arrow-length", 6.0,
 				       "arrow-width", 6.0,
 				       NULL);
+  setup_item_signals (polyline2);
 
   polyline3 = goo_canvas_polyline_new (root, FALSE, 2,
 				       356.0, 220.0,
@@ -888,6 +890,20 @@ setup_lines (GooCanvasItem *root)
 				       "arrow-length", 6.0,
 				       "arrow-width", 6.0,
 				       NULL);
+  setup_item_signals (polyline3);
+
+  /* Test polyline without any coords. */
+  polyline4 = goo_canvas_polyline_new (root, FALSE, 0,
+				       NULL);
+  setup_item_signals (polyline4);
+
+  /* Test polyline with 1 coord and arrows. */
+  polyline5 = goo_canvas_polyline_new (root, FALSE, 1,
+				       356.0, 220.0,
+				       "start-arrow", TRUE,
+				       "end-arrow", TRUE,
+				       NULL);
+  setup_item_signals (polyline5);
 }
 
 
@@ -909,15 +925,15 @@ setup_polygons (GooCanvasItem *root)
   points->coords[4] = 260.0;
   points->coords[5] = 350.0;
   pattern = create_stipple ("blue", stipple_data);
-  polyline1 = goo_canvas_item_new (root, GOO_TYPE_CANVAS_POLYLINE,
-				   "line-width", 1.0,
-				   "points", points,
-				   "fill_pattern", pattern,
-				   "stroke_color", "black",
-				   "close-path", TRUE,
-				   NULL);
+  polyline1 = goo_canvas_polyline_new (root, TRUE, 0,
+				       "line-width", 1.0,
+				       "points", points,
+				       "fill_pattern", pattern,
+				       "stroke_color", "black",
+				       NULL);
   cairo_pattern_destroy (pattern);
   goo_canvas_points_unref (points);
+  setup_item_signals (polyline1);
 
   points = goo_canvas_points_new (14);
   points->coords[0] = 270.0;
@@ -948,14 +964,14 @@ setup_polygons (GooCanvasItem *root)
   points->coords[25] = 410.0;
   points->coords[26] = 290.0;
   points->coords[27] = 330.0;
-  polyline2 = goo_canvas_item_new (root, GOO_TYPE_CANVAS_POLYLINE,
-				   "points", points,
-				   "fill_color", "tan",
-				   "stroke_color", "black",
-				   "line-width", 3.0,
-				   "close-path", TRUE,
-				   NULL);
+  polyline2 = goo_canvas_polyline_new (root, TRUE, 0,
+				       "points", points,
+				       "fill_color", "tan",
+				       "stroke_color", "black",
+				       "line-width", 3.0,
+				       NULL);
   goo_canvas_points_unref (points);
+  setup_item_signals (polyline2);
 }
 
 
@@ -965,7 +981,7 @@ make_anchor (GooCanvasItem *root, double x, double y)
   GooCanvasItem *group, *item;
   cairo_matrix_t transform = { 0.8, 0.2, -0.3, 0.5, x, y };
 
-  group = goo_canvas_group_new (root);
+  group = goo_canvas_group_new (root, NULL);
   goo_canvas_item_translate (group, x, y);
 #if 1
   g_object_set (group,
@@ -976,6 +992,7 @@ make_anchor (GooCanvasItem *root, double x, double y)
   item = goo_canvas_rect_new (group, -2.5, -2.5, 4, 4,
 			      "line-width", 1.0,
 			      NULL);
+  setup_item_signals (item);
 
   return group;
 }
@@ -983,6 +1000,7 @@ make_anchor (GooCanvasItem *root, double x, double y)
 static void 
 setup_texts (GooCanvasItem *root)
 {
+  GooCanvasItem *item;
   cairo_pattern_t *pattern;
   static guchar stipple_data[16] = {
     0, 0, 0, 255,   0, 0, 0, 0,   0, 0, 0, 0,     0, 0, 0, 255
@@ -990,33 +1008,36 @@ setup_texts (GooCanvasItem *root)
 
 #if 1
   pattern = create_stipple ("blue", stipple_data);
-  goo_canvas_text_new (make_anchor (root, 420, 20),
-		       "Anchor NW", 0, 0, -1, GTK_ANCHOR_NW,
-		       "font", "Sans Bold 24",
-		       "fill_pattern", pattern,
-		       NULL);
+  item = goo_canvas_text_new (make_anchor (root, 420, 20),
+			      "Anchor NW", 0, 0, -1, GTK_ANCHOR_NW,
+			      "font", "Sans Bold 24",
+			      "fill_pattern", pattern,
+			      NULL);
   cairo_pattern_destroy (pattern);
+  setup_item_signals (item);
 
-  goo_canvas_text_new (make_anchor (root, 470, 75),
-		       "Anchor center\nJustify center\nMultiline text\nb8bit text ÅÄÖåäö",
-		       0, 0, -1, GTK_ANCHOR_CENTER,
-		       "font", "monospace bold 14",
-		       "alignment", PANGO_ALIGN_CENTER,
-		       "fill_color", "firebrick",
-		       NULL);
+  item = goo_canvas_text_new (make_anchor (root, 470, 75),
+			      "Anchor center\nJustify center\nMultiline text\nb8bit text ÅÄÖåäö",
+			      0, 0, -1, GTK_ANCHOR_CENTER,
+			      "font", "monospace bold 14",
+			      "alignment", PANGO_ALIGN_CENTER,
+			      "fill_color", "firebrick",
+			      NULL);
+  setup_item_signals (item);
 #endif
 
 #if 0
-  goo_canvas_text_new (make_anchor (root, 590, 140),
-		       "Clipped text\nClipped text\nClipped text\nClipped text\nClipped text\nClipped text",
-		       0, 0, -1, GTK_ANCHOR_SE,
-		       "font", "Sans 12",
-		       /*"clip", TRUE,*/
-		       /*"clip_width", 50.0,*/
-		       /*"clip_height", 55.0,*/
-		       /*"x_offset", 10.0,*/
-		       "fill_color", "darkgreen",
-		       NULL);
+  item = goo_canvas_text_new (make_anchor (root, 590, 140),
+			      "Clipped text\nClipped text\nClipped text\nClipped text\nClipped text\nClipped text",
+			      0, 0, -1, GTK_ANCHOR_SE,
+			      "font", "Sans 12",
+			      /*"clip", TRUE,*/
+			      /*"clip_width", 50.0,*/
+			      /*"clip_height", 55.0,*/
+			      /*"x_offset", 10.0,*/
+			      "fill_color", "darkgreen",
+			      NULL);
+  setup_item_signals (item);
 #endif
 
 #if 1
@@ -1026,6 +1047,7 @@ setup_texts (GooCanvasItem *root)
 				  "font", "Sans 12",
 				  "fill_color", "goldenrod",
 				  NULL);
+  setup_item_signals (textitem);
 #endif
 }
 
@@ -1093,14 +1115,13 @@ plant_flower (GooCanvasItem *root, double x, double y, GtkAnchorType anchor)
   pattern = cairo_pattern_create_for_surface (surface);
   cairo_surface_destroy (surface);
 
-  image = goo_canvas_item_new (root, GOO_TYPE_CANVAS_IMAGE,
+  image = goo_canvas_image_new (root, NULL, x, y,
 			       "pattern", pattern,
-			       "x", x,
-			       "y", y,
 			       "width", w,
 			       "height", h,
 			       NULL);
   cairo_pattern_destroy (pattern);
+  setup_item_signals (image);
 }
 
 static void
@@ -1115,15 +1136,13 @@ setup_images (GooCanvasItem *root)
     {
       double w = gdk_pixbuf_get_width (im);
       double h = gdk_pixbuf_get_height (im);
-      image = goo_canvas_item_new (root, GOO_TYPE_CANVAS_IMAGE,
-				   "pixbuf", im,
-				   "x", 100.0 - w / 2,
-				   "y", 225.0 - h / 2,
+      image = goo_canvas_image_new (root, im, 100.0 - w / 2, 225.0 - h / 2,
 				   "width", w,
 				   "height", h,
 				   /* "anchor", GTK_ANCHOR_CENTER, */
 				   NULL);
       g_object_unref(im);
+      setup_item_signals (image);
     }
   else
     g_warning ("Could not find the toroid.png sample file");
@@ -1135,14 +1154,14 @@ setup_images (GooCanvasItem *root)
 }
 
 
-static GooCanvasModelSimple*
-create_model (void)
+static void
+setup_canvas (GooCanvas *canvas)
 {
-  GooCanvasModelSimple *canvas_model;
   GooCanvasItem *root;
 
-  canvas_model = goo_canvas_model_simple_new ();
-  root = goo_canvas_model_get_root_item (GOO_CANVAS_MODEL (canvas_model));
+  root = goo_canvas_get_root_item (canvas);
+  g_signal_connect (root, "button_press_event",
+		    (GtkSignalFunc) on_background_button_press, NULL);
 
   /* Setup canvas items */
 #if 1
@@ -1155,11 +1174,6 @@ create_model (void)
   setup_images (root);
   setup_invisible_texts (root);
 #endif
-#if 0
-  setup_widgets (root);
-#endif
-
-  return canvas_model;
 }
 
 
@@ -1173,7 +1187,7 @@ on_delete_event (GtkWidget *window,
 
 
 static GtkWidget*
-create_window (GooCanvasModelSimple *canvas_model)
+create_window ()
 {
   GtkWidget *window, *notebook;
 
@@ -1188,7 +1202,7 @@ create_window (GooCanvasModelSimple *canvas_model)
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
-			    create_canvas_primitives (canvas_model),
+			    create_canvas_primitives (),
 			    gtk_label_new ("Primitives"));
 #if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -1203,7 +1217,7 @@ create_window (GooCanvasModelSimple *canvas_model)
 #if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
 			    create_canvas_features (),
-			    gtk_label_new ("Features"));
+			    gtk_label_new ("Reparent"));
 #endif
 #if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -1230,6 +1244,16 @@ create_window (GooCanvasModelSimple *canvas_model)
 			    create_focus_page (),
 			    gtk_label_new ("Focus"));
 #endif
+#if 1
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    create_animation_page (),
+			    gtk_label_new ("Animation"));
+#endif
+#if 1
+  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+			    create_clipping_page (),
+			    gtk_label_new ("Clipping"));
+#endif
 
   return window;
 }
@@ -1238,19 +1262,12 @@ create_window (GooCanvasModelSimple *canvas_model)
 int
 main (int argc, char *argv[])
 {
-  GooCanvasModelSimple *canvas_model;
   GtkWidget *window;
 
   gtk_set_locale ();
   gtk_init (&argc, &argv);
 
-  canvas_model = create_model ();
-
-  /* Create 2 windows to show off multiple views. */
-  window = create_window (canvas_model);
-#if 1
-  window = create_window (canvas_model);
-#endif
+  window = create_window ();
 
   gtk_main ();
   return 0;
